# TetraControl2Connect

## Beschreibung
Dient der Verbindung von mehreren Connect-Organisationen mit einer TetraControl-Instanz. 

## Voraussetzungen
* TetraControl muss auf durchgehend laufen (am besten auf localhost, Websocket-Verbindung ist nicht verschlsselt!)
* In TetraControl muss der Webserver aktiviert sein
* In TetraControl muss ein enstprechender Benutzer mit ausreichenden Berechtigungen fr den Webserver angelegt werden.

## Hinweise
* Wenn Fahrzeuge oder Benutzer neu angelegt oder gendert werden, muss das Programm neu gestartet werden.
* Wir knnen nur Daten verarbeiten, wenn die ISSIs der Fahrzeuge und Benutzer in Connect korrekt hinterlegt sind.

## Konfiguration
Die Konfiguration wird in der Datei `appsettings.json` vorgenommen. Diese muss im gleichen Verzeichnis wie die Anwendung selbst liegen. 

### Abschnitt `TetraControlOptions`
Dies ist die Konfiguration der Verbindung zu TetraControl. 
* `TetraControlHost` -> Netzwerkadresse der TetraControl-Instanz (Hostname oder IP-Adresse)
* `TetraControlPort` -> Port von TetraControl
* `TetraControlUsername` -> Benutzername des in TetraControl angelegten Benutzers
* `TetraControlPassword` -> Passwort des in TetraControl angelegten Benutzers

### Abschnitt `ProgramOptions`
* Unter `SendVehicleStatus` kann eingestellt werden, ob Fahrzeugstatus bertragen werden (Standard: true)
* Unter `SendVehiclePositions` kann eingestellt werden, ob Fahrzeugpositionen bertragen werden (Standard: true)
* Unter `SendUserOperationStatus` kann eingestellt werden, ob Einsatzrckmeldungen von Benutzern bertragen werden (Standard: true)
* Unter `SendUserAvailability` kann eingestellt werden, ob Benutzer-Verfgbarkeiten (Taktische Verfgbarkeit) bertragen werden (Standard: true)
* Unter `SendAlarms` kann eingestellt werden, ob Callouts (Alarmierungen) ausgewertet und bertragen werden (Standard: true)
* Unter `UserAvailabilityLifetimeDays` kann die Anzahl der Tage eingestellt werden, wie lange eine Verfgbarkeit fr einen Benutzer gelten soll, bis sie zurckgesetzt wird.
* Unter `WebSocketReconnectTimeoutMinutes` kann die Zeitspanne in Minuten eingestellt werden, nachdem die WebSocket-Verbindung zu TetraControl neu hergestellt werden soll, nachdem in dieser keine Nachrichten von TetraControl empfangen worden sind. (Standard: 5 Minuten)
* Unter `PollForActiveOperationBeforeFallbackMaxRetryCount` kann die Anzahl der maximalen Versuche eingestellt werden, um nach einem aktiven Einsatz in Connect zu schauen, bevor ein Fallback-Alarm gegeben wird. (Standard: 4)
* Unter `PollForActiveOperationBeforeFallbackDelay` kann die Zeitverzgerung zwischen den Versuchen nach einem aktiven Einsatz in Connect zu schauen eingestellt werden. (Format HH:mm:ss, Standard 00:00:10)
* Unter `HeartbeatEndpointUrl` kann die Url angegeben werden, zu der im eingestellen Intervall ein HTTP-GET Request gesendet wird. (z.B. fr UptimeRobot, falls leer gelassen wird kein Heartbeat gesendet)
* Unter `HeartbeatInterval` kann das Intervall eingestellt werden, wie oft der Heartbeat-Aufruf erfolgen soll. (Format HH:mm:ss, wenn kein Wert wird kein Heartbeat gesendet)
* Unter `IgnoreStatus5` kann eingestellt werden, ob Status 5 (Sprechwunsch) ignoriert werden soll. (True/False, Standard: False)
* Unter `IgnoreStatus0` kann eingestellt werden, ob Status 0 (Priorisierter Sprechwunsch) ignoriert werden soll. (True/False, Standard: False)
* Unter `IgnoreStatus9` kann eingestellt werden, ob Status 9 (Fremdquittung / Sondersteuerungen) ignoriert werden soll. (True/False, Standard: False)
* Unter `AddPropertyForAlarmTexts` kann eingestellt werden, ob bei einer Einsatz-Aktualisierung nach Alarmeingang auch ein Zusatzfeld mit dem Alarmtext vom Pager erstellt werden soll
* Unter `UseFullyQualifiedSubnetAddressForConnect` kann eingestellt werden, ob die Darstellung der alarmierten Subnetzadressen ausfhrlich oder gekrzt erfolgen soll. (Bei True: z.B. "T2C(71234567_&01 - SBI"), bei False (standard): "SNA(&01)")
* Unter `IgnoreAlarmWithoutSubnetAddresses` kann eingestellt werden, ob Callouts (Alarme) ohne Subnetzadressen (also an die gesamte GSSI) ignoriert werden sollen (Standard: false)
* Unter `AcceptCalloutsForSirens` kann eingestellt werden, ob Callouts (Alarme) fr Sirenen (beginnend mit Steuerzeichen z.B. $2002) verarbeitet werden sollen (Standard: false)

### Abschnitt `StatusOptions`
Hier mssen die Statuswerte (Zahlenwert, nicht der Text) eingestellt werden, die vom Pager fr die einzelnen Status gesendet werden. (Bitte Melderkonfiguration beachten!)
* `AvailableStatus` -> Status "Verfgbar"
* `LimitedAvailableStatus` -> Status "Bedingt verfgbar"
* `NotAvailableStatus` -> Status "nicht verfgbar"
* `ComingStatus` -> Einsatzrckmeldung "komme"
* `NotComingStatus` -> Einsatzrckmeldung "komme nicht"
* `ComingLaterStatus` -> Einsatzrckmeldung "komme spter"

Wenn ein Status nicht verwendet wird, kann dort -1 eingetragen werden. Wichtig ist, dass dieser Status nicht an anderen Stellen in der Melderkonfiguration verwendet wird, um Missverstndnisse zu vermeiden.
Ab Version 2.1.9 ist es mglich, hier mehrere Statuswerte in folgendem Format einzutragen "123;456;789". (Ntzlich, um z.B. verschiedene Pager-Hersteller gemischt auszuwerten (z.B. Motorola und Airbus)).

### Abschnitt `SeverityOptions`
Hier wird der Umgang mit den Schweregraden von Alarmierungen gesteuert. (optional)
* `UseServerityTranslationAsKeyword` -> Schweregrad-bersetzung als Stichwort fr Fallback-Einstze verwenden (anstelle von "ALARM") (Standard: True mit den Standardwerten von Hessen)
* `SeverityTranslations` -> Da die Schweregrad-Texte von der Melderprogrammierung abhngig sind und nur die Zahl von TetraControl bermittelt wird, knnen hier die bersetzungen der Schweregrade eingetragen werden. (Standard: Hessen) Falls keine bersetzung gefunden wird, wird "ALARM" genommen

### Abschnitt `SirenCalloutOptions`
Hier wird der Umgang mit den Steuerzeichen von Sirenenalarmierungen gesteuert. (optional)
* `UseSirenCodeTranslationAsKeyword` -> Steuercode-bersetzung als Stichwort fr Fallback-Einstze verwenden (anstelle von "ALARM" bzw. der Schweregrad-bersetzung) (Standard: False)
* `SirenCodeTranslations` -> Da die Steuerzeichen-Texte vom Bundesland abhngig sind, knnen hier die bersetzungen der Steuerzeichen eingetragen werden. (Standard: Hessen) Falls keine bersetzung gefunden wird, wird "ALARM" bzw. wenn konfiguriert die Schweregrad-bersetzung genommen

### Abschnitt `ConnectOptions`
Hier mssen die API-Keys aus Connect eingetragen werden. Unter `Name` kann ein frei whlbarer Bezeichner hinterlegt werden, um die Schlssel besser zuordnen zu knnen.
Eintragen mehrerer Schlssel erfolgt nach der JSON-Syntax. (Siehe https://developer.mozilla.org/de/docs/Learn/JavaScript/Objects/JSON#arrays_als_json)
Die SubnetAddresses sind aktuell nur fr bestimmte Feuerwehren relevant und mssen nicht ausgefllt werden. Unter `AlarmDirectly` kann pro Schleife eingestellt werden, ob ein Alarm direkt nach Pagerauslsung nach Connect hochgeladen werden soll. 
Dann wird fr diese Schleife nicht versucht, einen vorhandenen Einsatz zu aktualisieren.
```json
"ConnectOptions": {
    "Sites": [
        {
            "Name": "Organisation1",
            "Key": "<<KEY1>>",
            "SubnetAddresses": [
                {
                    "Name": "Schleife1",
                    "GSSI": "12345678",
                    "SNA": "&01",
                    "AlarmDirectly": false
                },
                {
                    "Name": "Schleife2",
                    "GSSI": "12345678",
                    "SNA": "&01",
                    "AlarmDirectly": false
                }
            ]
        },
        {
            "Name": "Organisation2",
            "Key": "<<KEY2>>",
            "SubnetAddresses": [
                {
                    "Name": "Schleife1",
                    "GSSI": "12345678",
                    "SNA": "&01",
                    "AlarmDirectly": false
                },
                {
                    "Name": "Schleife2",
                    "GSSI": "12345678",
                    "SNA": "&01",
                    "AlarmDirectly": false
                }
            ]
        }
    ]
}
```

## Funktionsweise
Das Programm verbindet sich ber eine Websocket-Verbindung mit dem Webserver von TetraControl und verarbeitet Positionsdaten, Statusdaten und SDS. Funkgesprche werden ignoriert.

## Vollstndies Konfigurationsbeispiel:
```json
{
  "TetraControlOptions": {
    "TetraControlHost": "localhost",
    "TetraControlPort": 80,
    "TetraControlUsername": "Connect",
    "TetraControlPassword": "Connect"
  },
  "ProgramOptions": {
    "SendVehicleStatus": true,
    "SendVehiclePositions": true,
    "SendUserOperationStatus": true,
    "SendUserAvailability": true,
    "SendAlarms": true,
    "UserAvailabilityLifetimeDays": 7,
    "WebSocketReconnectTimeoutMinutes": 5,
    "PollForActiveOperationBeforeFallbackMaxRetryCount": 4,
    "PollForActiveOperationBeforeFallbackDelay": "00:00:10",
    "HeartbeatEndpointUrl": "",
    "HeartbeatInterval": "00:10:00",
    "IgnoreStatus5": false,
    "IgnoreStatus0": false,
    "IgnoreStatus9": false,
    "AddPropertyForAlarmTexts": false,
    "UseFullyQualifiedSubnetAddressForConnect": false,
    "IgnoreAlarmWithoutSubnetAddresses": false
  },
  "StatusOptions": {
    "AvailableStatus": 15,
    "LimitedAvailableStatus": -1,
    "NotAvailableStatus": 0,
    "ComingStatus": "32768;57345",
    "NotComingStatus": "32769;57344",
    "ComingLaterStatus": -1
  },
  "SeverityOptions": {
    "UseServerityTranslationAsKeyword": true,
    "SeverityTranslations": {
      "1": "Information",
      "2": "Einsatzabbruch",
      "3": "Bereitschaft",
      "4": "Krankentransport",
      "5": "Rettungsdienst R-O",
      "7": "Hilfeleistung normal",
      "8": "Feuer normal",
      "9": "Rettungsdienst R-1",
      "10": "Rettungsdienst R-2",
      "11": "Hilfeleistung dringend",
      "12": "Feuer dringend",
      "13": "Groalarm",
      "15": "KatS-Alarm"
    }
  },
  "SirenCalloutOptions": {
    "UseSirenCodeTranslationAsKeyword": false,
    "SirenCodeTranslations": {
      "$2000": "Warnung der Bevlkerung",
      "$2001": "Entwarnung",
      "$2002": "Feueralarm",
      "$2003": "Probealarm"
    }
  },
  "ConnectOptions": {
    "Sites": [
      {
        "Name": "Organisation1",
        "Key": "<<KEY>>",
        "SubnetAddresses": [
          {
            "Name": "Schleife1",
            "GSSI": "12345678",
            "SNA": "&01",
            "AlarmDirectly": false
          }
        ]
      }
    ]
  }
}
```

## Copyright
Copyright Feuer Software GmbH, Karlsbader Str. 16, Eschborn

Internet: https://feuersoftware.com
Mail: info@feuersoftware.com

Alle Rechte vorbehalten.